<?php

namespace Drupal\file_compressor_field\Plugin\FileCompressor;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Password\PasswordGeneratorInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\file_compressor_field\Plugin\FileCompressorBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Implements GZip compressor for File Compressor field.
 *
 * @FileCompressor(
 *   id = "gzip_zlib",
 *   admin_label = @Translation("GZip (Zlib)"),
 *   extension = "tar.gz"
 * )
 */
class FileCompressorGZip extends FileCompressorBase implements ContainerFactoryPluginInterface {

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The password generator service.
   *
   * @var \Drupal\Core\Password\PasswordGeneratorInterface
   */
  protected $passwordGenerator;

  /**
   * The logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new FileCompressorGZip object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Password\PasswordGeneratorInterface $password_generator
   *   The password generator service.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The logger service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   */
  final public function __construct(array $configuration,
                                    $plugin_id,
                                    $plugin_definition,
                                    FileSystemInterface $file_system,
                                    PasswordGeneratorInterface $password_generator,
                                    LoggerChannelInterface $logger,
                                    ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->fileSystem = $file_system;
    $this->passwordGenerator = $password_generator;
    $this->logger = $logger;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container,
                                array $configuration,
                                $plugin_id,
                                $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('file_system'),
      $container->get('password_generator'),
      $container->get('logger.factory')->get('file_compressor_field'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function generateCompressedFile($file_uri, $files) {
    $full_file_uri = $this->fileSystem->realpath($file_uri);
    $tmp = 'temporary://file_compressor_field' . time() . $this->passwordGenerator->generate();
    $this->fileSystem->mkdir($tmp);
    foreach ($files as $file) {
      $this->fileSystem->copy($file, $tmp, FileSystemInterface::EXISTS_REPLACE);
    }
    $method = $this->configFactory->getEditable('file_compressor_field.settings')->get('field_gzip_method') ?? 'php';

    // Use PHP's Archive_Tar class.
    if ($method === 'php') {
      $tar = new \Archive_Tar($full_file_uri, 'gz');
      $result = $tar->createModify($tmp, '', $tmp);
      if ($result !== TRUE) {
        $this->logger->error('Error compressing file %file. Error: %error', [
          '%file' => $full_file_uri,
          '%error' => $result,
        ]);
        return FALSE;
      }
    }

    // System / command line.
    elseif ($method === 'system') {
      $command = 'tar -czf ' . escapeshellarg($full_file_uri) . ' ' . escapeshellarg($tmp);
      exec($command, $output, $return);
      if ($return !== 0) {
        $this->logger->error('Error compressing file %file. Command: %command ...failed with return status %return. Output: %output', [
          '%file' => $full_file_uri,
          '%command' => $command,
          '%return' => $return,
          '%output' => implode("\n", $output),
        ]);
        return FALSE;
      }
    }

    $this->fileSystem->deleteRecursive($tmp);
    return TRUE;
  }

}
