<?php

namespace Drupal\file_compressor_field\Plugin\FileCompressor;

use Drupal\Core\Config\ConfigFactoryInterface;
use Drupal\Core\File\FileSystemInterface;
use Drupal\Core\Logger\LoggerChannelInterface;
use Drupal\Core\Plugin\ContainerFactoryPluginInterface;
use Drupal\file_compressor_field\Plugin\FileCompressorBase;
use Symfony\Component\DependencyInjection\ContainerInterface;

/**
 * Implements Zip compressor for File Compressor field.
 *
 * @FileCompressor(
 *   id = "zip_built_in",
 *   admin_label = @Translation("Zip (Built in)"),
 *   extension = "zip"
 * )
 */
class FileCompressorZip extends FileCompressorBase implements ContainerFactoryPluginInterface {

  /**
   * The file system service.
   *
   * @var \Drupal\Core\File\FileSystemInterface
   */
  protected $fileSystem;

  /**
   * The logger service.
   *
   * @var \Drupal\Core\Logger\LoggerChannelInterface
   */
  protected $logger;

  /**
   * The configuration factory.
   *
   * @var \Drupal\Core\Config\ConfigFactoryInterface
   */
  protected $configFactory;

  /**
   * Constructs a new FileCompressorZip object.
   *
   * @param array $configuration
   *   A configuration array containing information about the plugin instance.
   * @param string $plugin_id
   *   The plugin_id for the plugin instance.
   * @param mixed $plugin_definition
   *   The plugin implementation definition.
   * @param \Drupal\Core\File\FileSystemInterface $file_system
   *   The file system service.
   * @param \Drupal\Core\Logger\LoggerChannelInterface $logger
   *   The logger service.
   * @param \Drupal\Core\Config\ConfigFactoryInterface $config_factory
   *   The configuration factory.
   */
  final public function __construct(array $configuration,
                                    $plugin_id,
                                    $plugin_definition,
                                    FileSystemInterface $file_system,
                                    LoggerChannelInterface $logger,
                                    ConfigFactoryInterface $config_factory) {
    parent::__construct($configuration, $plugin_id, $plugin_definition);
    $this->fileSystem = $file_system;
    $this->logger = $logger;
    $this->configFactory = $config_factory;
  }

  /**
   * {@inheritdoc}
   */
  public static function create(ContainerInterface $container,
                              array $configuration,
                              $plugin_id,
                              $plugin_definition) {
    return new static(
      $configuration,
      $plugin_id,
      $plugin_definition,
      $container->get('file_system'),
      $container->get('logger.factory')->get('file_compressor_field'),
      $container->get('config.factory')
    );
  }

  /**
   * {@inheritdoc}
   */
  public function generateCompressedFile($file_uri, $files) {
    $full_file_uri = $this->fileSystem->realpath($file_uri);
    $method = $this->configFactory->getEditable('file_compressor_field.settings')->get('field_zip_method') ?? 'php';

    // Use PHP's ZipArchive class.
    if ($method === 'php') {
      $zip = new \ZipArchive();

      // Since libzip 1.6.0, a empty file is not a valid archive any longer.
      if (!empty($full_file_uri) && $zip->open($full_file_uri, \ZIPARCHIVE::CREATE | \ZIPARCHIVE::OVERWRITE)) {
        $zip->setArchiveComment('Generated by File Compressor field for Drupal.');
        // http://drupal.org/node/1234282 - duplicate filenames.
        foreach ($files as $file) {
          $path = $this->fileSystem->realpath($file);
          if (file_exists($file)) {
            if (!$zip->addFile($path, basename($path))) {
              $this->logger->error('Failed to add !file to Zip.', ['!file' => $file]);
            }
          }
          else {
            $this->logger->error('Failed to locate !file.', ['!file' => $file]);
          }
        }
        if (!$zip->close()) {
          $this->logger->error('Error saving file: !status_string', ['!status_string' => $zip->getStatusString()]);
          return FALSE;
        }
        return TRUE;
      }
    }

    // System / command line.
    elseif ($method === 'system') {
      // Build command.
      if (!empty($full_file_uri)) {
        $command = 'zip -j ' . escapeshellarg($full_file_uri);
        foreach ($files as $file) {
          $command .= ' ' . escapeshellarg($this->fileSystem->realpath($file));
        }
        // Execute command.
        exec($command, $output, $return);
        if ($return !== 0) {
          $this->logger->error('Error saving file %file_uri. Command: %command ...failed with return status %return. Output: %output', [
            '%file' => $file_uri,
            '%command' => $command,
            '%return' => $return,
            '%output' => implode("\n", $output),
          ],
          );
          return FALSE;
        }
        return TRUE;
      }
    }

    return FALSE;
  }

}
